package Source::Bundle;

use Cwd;
use Carp;
use strict;
use warnings;

#use File::Path qw(make_path remove_tree);
#use File::Temp qw/tempfile tempdir/;
use File::Basename;

our $VERSION=0.01;

sub new {
    my $class = shift;
    my $self = {
	-verbose => 0,
	@_
    };
    my $file = Cwd::abs_path($self->{-file});
    Carp::cluck "Source::Bundle: no $file:\n" if not -e $file;
    return unless -e $file;
    $self->{-file}=$file;
    bless $self, $class;
    return $self;
}

# compat interface; to kill
sub extract_files {
    my ($self, $list)=@_;
    return $self->extract(-list=>$list);
}

sub filelist {
    my ($self)=@_;
    my $listref=$self->{-filelist};
    return $listref if $listref;
    my $file=$self->{-file};
    my $listcmd=$self->filelist_cmd($file);
    my $leading_dot=0;
    my %dirs;
    my %files;
    if ($listcmd) {
	open (my $fn, "$listcmd |") || die "can't run $listcmd: $!";
	my $name;
	while ($name=<$fn>) {
	    chomp $name;
	    $leading_dot=1 if $name =~ s!^\./!!;
	    next if $name eq ''; # ./ entry sometimes mixes in tar
	    next if $name=~m!/$!; # ignore dir; valid for both tar and zipinfo
	    $files{$name}=1;
	    $dirs{dirname($name)}=1;
	}
	close $fn;
    }
    my @filelist=grep {not $dirs{$_}} keys %files;
    $self->{-filelist}=\@filelist;
    $self->{__filelist_leading_dot}=$leading_dot;
    return \@filelist;
}

sub depth {
    my ($self)=@_;
    my $ref=$self->{-depth};
    return @$ref if $ref;
    my $filelist=$self->filelist();
    my $depthlist=&_depth_from_filelist($filelist);
    $self->{-depth}=$depthlist;
    return @$depthlist;
}

# used in source analyser
sub _depth_from_filelist {
   my $filelist=shift;
    my @depthlist;
    if (@$filelist) {
	@depthlist=split ('/',$filelist->[0],-1);
	pop @depthlist;
	if (@depthlist) {
	  S1:
	    for (my $i=1; $i<@$filelist; $i++) {
		my @second=split ('/',$filelist->[$i],-1);
	      S2:
		for (my $j=0;$j<@depthlist;$j++) {
		    unless (defined $second[$j] and $second[$j] eq $depthlist[$j]) {
			$#depthlist=$j-1;
			last S1 if $#depthlist<0;
			last S2;
		    }
		}
	    }
	}
    }
    return \@depthlist;
}


sub depth_prefix {
    my ($self)=@_;
    my @depth=$self->depth();
    my $prefix='';
    return $prefix if not @depth;
    my $depth_prefix=join('/',@depth).'/';
}

# used in RPM::Source::Transformation::Resource::InputBundle
sub rebase_to_depth {
    my ($tarball)=@_;
    my @depth=$tarball->depth();
    my $filelist=$tarball->filelist();
    return &_rebase_filelist($filelist,\@depth);
}

# used in RPM::Source::Dependency::Analyzer::Collector
sub _rebase_filelist {
    my ($filelist,$depth)=@_;
    return $filelist if not @$depth;
    my $depth_prefix=join('/',@$depth);
    my @rebased_list;
    foreach my $file (@$filelist) {
	my $rebased_file=$file;
	$rebased_file=~s,^\Q$depth_prefix\E/,,;
	push @rebased_list, $rebased_file;
    }
    return \@rebased_list;
}

1;
