package Source::Bundle::Archive;

use Cwd;
use Carp;
use strict;
use warnings;

use File::Temp qw/tempfile tempdir/;
use Source::Bundle;
use Source::Bundle::File;

our @ISA=qw/Source::Bundle/;
our $VERSION=0.001;

sub new {
    my $class = shift;
    my $self=$class->SUPER::new(@_);
    my (@unpackcmd, $listcmd);
    my $file = $self->{-file};
    if ($file=~/\.(?:tgz|tbz2?|txz|tar|tar\.(?:bz2?|gz|Z|xz|lzma))$/i) {
	# --overwrite does not work due to file permissions;
	# both --keep-old-files and omitting args seem to work
	# see pmtools-1.10.tar.gz in perl-pmtools
	#@unpackcmd= ('tar','--overwrite','-x','-f');
	#@unpackcmd= ('tar','--keep-old-files','-x','-f');
	@unpackcmd= ('tar','-x','-f');
	$listcmd= 'tar tf';
    } elsif ($file=~/\.zip$/i) {
	$listcmd= 'zipinfo -1';
	@unpackcmd= ('unzip','-q');
    } elsif ($file=~/\.rar$/i) {
	$listcmd= 'unrar lb';
	@unpackcmd= ('unrar','x');
    } elsif ($file=~/\.(?:jpg|jpeg|gif|svg|xpm|png # images
			|dvi|ps|pdf # publishing
			|otf|ttf|bdf # fonts
			|dict\.dz|idx # dictionary
			|rom|asc|sig|info|[13678])(?:\.(gz|bz2|xz|lzma))?$/ix) {
	return Source::Bundle::File->new(@_,-file=>$file);
	# unknown/unsupported archive or just plain file?
    } elsif (`file "$file"` =~ /(?:text|script)/) {
	return Source::Bundle::File->new(@_,-file=>$file);
    } else {
	warn "unsupported type of archive: $file\n";
    }
    $self->{-listcmd}=$listcmd;
    $self->{-unpackcmd}=\@unpackcmd;
    bless $self, $class;
    return $self;
}

# options: -list =>\@list_to_extract
sub extract {
    my $self = shift;
    my %opt=@_;
    my $list=$opt{-list};

    my $curdir = cwd();
    my $tmpdir = File::Temp->newdir('TEMPLATE' => 'archive-extract-XXXXXXXXXX');
    my $file=$self->{-file};
    chdir $tmpdir;
    # sometimes ./ is prepended to the path, as in GeoIP-1.4.8.tar.gz
    # in that case it should be restored with map {'./'.$_}, otherwise
    # tar: GeoIP-1.4.8/configure.in: not found in archive
    if ($list and not system (@{$self->{-unpackcmd}}, $file, $self->{__filelist_leading_dot} ? map {'./'.$_} @$list : @$list)==0) {
	warn 'extracting '.join(' ',@$list)." from $file failed. Unpacking all...";
	# unpack 
	$list=undef;
    }
    if (not $list) {
	if (system (@{$self->{-unpackcmd}}, $file)!=0) {
	    chdir $curdir;
	    die "Unpacking $file failed.\n";
	}
    }
    chdir $curdir;
    return $tmpdir;
}

sub filelist_cmd {
    my ($self,$file)=@_;
    my $listcmd=$self->{-listcmd};
    return unless $listcmd;
    return "$listcmd '$file'";
}

1;
